#ifndef stracciatella_h
#define stracciatella_h

/* Generated with cbindgen:0.9.0 */

/* DO NOT MODIFY MANUALLY! Changes will be discarded during build. */

#include <cstdarg>
#include <cstdint>
#include <cstdlib>
#include <new>

/// Enum to represent log levels in the application
enum class LogLevel {
  Error = 0,
  Warn = 1,
  Info = 2,
  Debug = 3,
  Trace = 4,
};

/// Enum used to specify scaling quality for scaling up graphics
enum class ScalingQuality {
  /// Use linear interpolation
  LINEAR,
  /// Use nearest neighbor interpolation
  NEAR_PERFECT,
  /// Scale up to the nearest multiple of 640x480 and use nearest neighbor interpolation
  PERFECT,
};

/// Enum for the vanilla game version that is used to run the game
enum class VanillaVersion {
  DUTCH,
  ENGLISH,
  FRENCH,
  GERMAN,
  ITALIAN,
  POLISH,
  RUSSIAN,
  RUSSIAN_GOLD,
};

/// Struct that is used to store the engines configuration parameters
struct EngineOptions;

/// Thread safe library database.
struct LibraryDB;

/// Library file.
/// Provides access to the data of a library entry through Read and Seek.
struct LibraryFile;

/// A wrapper around `Vec<CString>` for C.
struct VecCString;

extern "C" {

/// Adds a library to the end of the library database.
/// Returns true is successful, false otherwise.
/// Sets the rust error.
bool LibraryDB_AddLibrary(LibraryDB *ldb, const char *data_dir, const char *library);

/// Destructor.
/// The caller is no longer responsible for the memory of the library database.
void LibraryDB_Delete(LibraryDB *ldb);

/// Constructor.
/// The caller is responsible for the memory of the library database.
LibraryDB *LibraryDB_New();

/// Closes a library database file.
/// The caller is no longer responsible for the library file memory.
void LibraryFile_Close(LibraryFile *file);

/// Gets the current position in a library database file.
uint64_t LibraryFile_GetPos(LibraryFile *file);

/// Gets the size of a library database file.
uint64_t LibraryFile_GetSize(LibraryFile *file);

/// Opens a library database file for reading.
/// Returns the file on success and null on error.
/// The caller is responsible for the library file memory.
/// Sets the rust error.
LibraryFile *LibraryFile_Open(LibraryDB *ldb, const char *path);

/// Reads from a library database file.
/// Sets the rust error.
bool LibraryFile_Read(LibraryFile *file, uint8_t *buffer, size_t buffer_length);

/// Seeks a library database file.
/// `from` expects FileSeekMode values (see src/sgp/SGPFile.h)
/// Sets the rust error.
bool LibraryFile_Seek(LibraryFile *file, int64_t distance, int from);

/// Initializes the logger
void Logger_Init(const char *log_file);

/// Log with custom metadata
void Logger_LogWithCustomMetadata(LogLevel level, const char *message, const char *target);

/// Set log level
void Logger_SetLevel(LogLevel level);

/// Returns true if it was able to find path relative to base.
/// Makes caseless searches one component at a time.
bool check_if_relative_path_exists(const char *base, const char *path, bool caseless);

/// Clears `EngineOptions.mods`.
void clear_mods(EngineOptions *ptr);

/// Creates `EngineOptions` with the provided command line arguments.
/// Loads values from `(stracciatella_home)/ja2.json`, creating it if it does not exist.
/// The caller is responsible for the returned memory.
EngineOptions *create_engine_options(const char *const *array, size_t length);

/// Converts the launcher executable path to the game executable path.
/// The executable is assumed to be in the same directory as the launcher.
/// The caller is responsible for the returned memory.
char *find_ja2_executable(const char *launcher_path_ptr);

/// Finds a path relative to the assets directory.
/// If path is null, it returns the assets directory.
/// If test_exists is true and the path does not exist, it returns null.
/// The caller is responsible for the returned memory.
char *find_path_from_assets_dir(const char *path, bool test_exists);

/// Finds a path relative to the stracciatella home directory.
/// If path is null, it finds the stracciatella home directory.
/// If test_exists is true, it makes sure the path exists.
/// The caller is responsible for the returned memory.
char *find_path_from_stracciatella_home(const char *path, bool test_exists);

/// Deletes `EngineOptions`.
void free_engine_options(EngineOptions *ptr);

/// Deletes a CString.
/// The caller is no longer responsible for the memory.
void free_rust_string(char *s);

/// Returns a list of available mods.
/// The caller is responsible for the returned memory.
VecCString *get_available_mods();

/// Gets `EngineOptions.brightness`.
float get_brightness(const EngineOptions *ptr);

/// Gets the target index of `EngineOptions.mods`.
/// The caller is responsible for the returned memory.
char *get_mod(const EngineOptions *ptr, uint32_t index);

/// Gets the length of `EngineOptions.mods`.
uint32_t get_number_of_mods(const EngineOptions *ptr);

/// Gets the width of `EngineOptions.resolution`.
uint16_t get_resolution_x(const EngineOptions *ptr);

/// Gets the height of `EngineOptions.resolution`.
uint16_t get_resolution_y(const EngineOptions *ptr);

/// Gets `EngineOptions.resource_version`.
VanillaVersion get_resource_version(const EngineOptions *ptr);

/// Gets the string represntation of the `VanillaVersion` value.
/// The caller is responsible for the returned memory.
char *get_resource_version_string(VanillaVersion version);

/// Returns a copy of the thread local rust error or null.
/// The caller is responsible for the memory.
char *get_rust_error();

/// Gets `EngineOptions.scaling_quality`.
ScalingQuality get_scaling_quality(const EngineOptions *ptr);

/// Gets the string representation of the `ScalingQuality` value.
/// The caller is responsible for the returned memory.
char *get_scaling_quality_string(ScalingQuality quality);

/// Gets the `EngineOptions.stracciatella_home` path.
/// The caller is responsible for the returned memory.
char *get_stracciatella_home(const EngineOptions *ptr);

/// Gets the `EngineOptions.vanilla_game_dir` path.
/// The caller is responsible for the returned memory.
char *get_vanilla_game_dir(const EngineOptions *ptr);

/// Guesses the resource version from the contents of the game directory.
/// Returns a VanillaVersion value if it was sucessful, -1 otherwise.
int guess_resource_version(const char *gamedir);

/// Adds a mod to `EngineOptions.mods`.
void push_mod(EngineOptions *ptr, const char *name);

/// Sets `EngineOptions.brightness`.
void set_brightness(EngineOptions *ptr, float brightness);

/// Sets `EngineOptions.resolution`.
void set_resolution(EngineOptions *ptr, uint16_t x, uint16_t y);

/// Sets `EngineOptions.resource_version`.
void set_resource_version(EngineOptions *ptr, VanillaVersion res);

/// Sets `EngineOptions.scaling_quality`.
void set_scaling_quality(EngineOptions *ptr, ScalingQuality scaling_quality);

/// Sets `EngineOptions.start_in_fullscreen`.
void set_start_in_fullscreen(EngineOptions *ptr, bool val);

/// Sets `EngineOptions.start_without_sound`.
void set_start_without_sound(EngineOptions *ptr, bool val);

/// Sets the `EngineOptions.vanilla_game_dir` path.
void set_vanilla_game_dir(EngineOptions *ptr, const char *game_dir_ptr);

/// Gets `EngineOptions.run_editor`.
bool should_run_editor(const EngineOptions *ptr);

/// Gets `EngineOptions.run_unittests`.
bool should_run_unittests(const EngineOptions *ptr);

/// Gets `EngineOptions.show_help`.
bool should_show_help(const EngineOptions *ptr);

/// Gets `EngineOptions.start_in_debug_mode`.
bool should_start_in_debug_mode(const EngineOptions *ptr);

/// Gets `EngineOptions.start_in_fullscreen`.
bool should_start_in_fullscreen(const EngineOptions *ptr);

/// Gets `EngineOptions.start_in_window`.
bool should_start_in_window(const EngineOptions *ptr);

/// Gets `EngineOptions.start_without_sound`.
bool should_start_without_sound(const EngineOptions *ptr);

/// Deletes the vector.
void vec_c_string_delete(VecCString *vec);

/// Returns the string at the target index.
/// The caller is responsible for the returned memory.
char *vec_c_string_get(VecCString *vec, size_t index);

/// Returns the vector length.
size_t vec_c_string_len(VecCString *vec);

/// Writes `EngineOptions` to `(stracciatella_home)/ja2.json`.
/// Returns true on success, false otherwise.
bool write_engine_options(EngineOptions *ptr);

} // extern "C"

#endif // stracciatella_h
